==> mic2_0.19.orig.tar.gz: mic2-0.19/mic/appcreate/partitionedfs.py <==
#
# partitionedfs.py: partitioned files system class, extends fs.py
#
# Copyright 2007-2008, Red Hat  Inc.
# Copyright 2008, Daniel P. Berrange
# Copyright 2008,  David P. Huff
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import os.path
import glob
import shutil
import subprocess
import logging

from mic.imgcreate.errors import *
from mic.imgcreate.fs import *


class PartitionedMount(Mount):
    def __init__(self, disks, mountdir, skipformat = False):
        Mount.__init__(self, mountdir)
        self.disks = {}
        for name in disks.keys():
            self.disks[name] = { 'disk': disks[name],  # Disk object
                                 'mapped': False, # True if kpartx mapping exists
                                 'numpart': 0, # Number of allocate partitions
                                 'partitions': [], # indexes to self.partitions
                                 'extended': 0, # Size of extended partition
                                 'offset': 0 } # Offset of next partition

        self.partitions = []
        self.mapped = False
        self.mountOrder = []
        self.unmountOrder = []
        self.parted="/usr/sbin/parted"
        if not os.path.exists(self.parted):
            self.parted="/sbin/parted"
        self.skipformat = skipformat

    def add_partition(self, size, disk, mountpoint, fstype = None, boot = False):
        self.partitions.append({'size': size,
                                'mountpoint': mountpoint, # Mount relative to chroot
                                'fstype': fstype,
                                'disk': disk,  # physical disk name holding partition
                                'device': None, # kpartx device node for partition
                                'mount': None, # Mount object
                                'num': None, # Partition number
                                'boot': boot}) # Bootable flag

    def __format_disks(self):
        dev_null = os.open("/dev/null", os.O_WRONLY)

        logging.debug("Assigning partitions to disks")
        for n in range(len(self.partitions)):
            p = self.partitions[n]

            if not self.disks.has_key(p['disk']):
                raise MountError("No disk %s for partition %s" % (p['disk'], p['mountpoint']))

            d = self.disks[p['disk']]
            d['numpart'] += 1
            if d['numpart'] > 3:
                # Increase allocation of extended partition to hold this partition
                d['extended'] += p['size']
                p['type'] = 'logical'
                p['num'] = d['numpart'] + 1
            else:
                p['type'] = 'primary'
                p['num'] = d['numpart']

            p['start'] = d['offset']
            d['offset'] += p['size']
            d['partitions'].append(n)
            logging.debug("Assigned %s to %s%d at %d at size %d" % (p['mountpoint'], p['disk'], p['num'], p['start'], p['size']))

        if self.skipformat:
            logging.debug("Skip disks format")
            return
        logging.debug("Formatting disks")
        for dev in self.disks.keys():
            d = self.disks[dev]
            logging.debug("Initializing partition table for %s" % (d['disk'].device))
            # FIXME
            rc = subprocess.call([self.parted, "-s", d['disk'].device, "mklabel", "msdos"],
                                 stdout = dev_null, stderr = dev_null)

            # XXX disabled return code check because parted always fails to
            # reload part table with loop devices. Annoying because we can't
            # distinguish this failure from real partition failures :-(
            if rc != 0 and 1 == 0:
                os.close(dev_null)
                raise MountError("Error writing partition table on %s" % d['disk'].device)

        # XXX we should probably work in cylinder units to keep fdisk happier..
        start = 0
        logging.debug("Creating partitions")
        for p in self.partitions:
            d = self.disks[p['disk']]
            if p['num'] == 5:
                logging.debug("Added extended part at %d of size %d" % (p['start'], d['extended']))
                rc = subprocess.call([self.parted, "-s", d['disk'].device, "mkpart", "extended",
                                     "%dM" % p['start'], "%dM" % (p['start'] + d['extended'])],
                                     stdout = dev_null, stderr = dev_null)

            logging.debug("Add %s part at %d of size %d" % (p['type'], p['start'], p['size']))
            rc = subprocess.call([self.parted, "-s", d['disk'].device, "mkpart",
                                 p['type'], "%dM" % p['start'], "%dM" % (p['start']+p['size'])],
                                 stdout = dev_null, stderr = dev_null)

            if p['boot']:
                rc = subprocess.call([self.parted, "-s", d['disk'].device, "set", "%d" % p['num'], "boot", "on"],
                                     stdout = dev_null, stderr = dev_null)
            # XXX disabled return code check because parted always fails to
            # reload part table with loop devices. Annoying because we can't
            # distinguish this failure from real partition failures :-(
            if rc != 0 and 1 == 0:
                os.close(dev_null)
                raise MountError("Error creating partition on %s" % d['disk'].device)
        os.close(dev_null)

    def __map_partitions(self):
        """Load it if dm_snapshot isn't loaded"""
        load_module("dm_snapshot")

        dev_null = os.open("/dev/null", os.O_WRONLY)
        for dev in self.disks.keys():
            d = self.disks[dev]
            if d['mapped']:
                continue

            logging.debug("Running kpartx on %s" % d['disk'].device )
            kpartx = subprocess.Popen(["/sbin/kpartx", "-l", "-v", d['disk'].device],
                                      stdout=subprocess.PIPE, stderr=dev_null)

            kpartxOutput = kpartx.communicate()[0].strip().split("\n")

            if kpartx.returncode:
                os.close(dev_null)
                raise MountError("Failed to query partition mapping for '%s'" %
                                 d.device)

            # Strip trailing blank and mask verbose output
            i = 0
            while i < len(kpartxOutput) and kpartxOutput[i][0:4] != "loop":
               i = i + 1
            kpartxOutput = kpartxOutput[i:]

            # Quick sanity check that the number of partitions matches
            # our expectation. If it doesn't, someone broke the code
            # further up
            if len(kpartxOutput) != d['numpart']:
                os.close(dev_null)
                raise MountError("Unexpected number of partitions from kpartx: %d != %d" %
                                 (len(kpartxOutput), d['numpart']))

            for i in range(len(kpartxOutput)):
                line = kpartxOutput[i]
                newdev = line.split()[0]
                mapperdev = "/dev/mapper/" + newdev
                loopdev = d['disk'].device + newdev[-1]

                logging.debug("Dev %s: %s -> %s" % (newdev, loopdev, mapperdev))
                pnum = d['partitions'][i]
                self.partitions[pnum]['device'] = loopdev

                # grub's install wants partitions to be named
                # to match their parent device + partition num
                # kpartx doesn't work like this, so we add compat
                # symlinks to point to /dev/mapper
                if os.path.exists(loopdev):
                    os.unlink(loopdev)
                os.symlink(mapperdev, loopdev)

            logging.debug("Adding partx mapping for %s" % d['disk'].device)
            rc = subprocess.call(["/sbin/kpartx","-v", "-a", d['disk'].device],
                                 stdout=dev_null, stderr=dev_null)
            if rc != 0:
                os.close(dev_null)
                raise MountError("Failed to map partitions for '%s'" %
                                 d['disk'].device)
            d['mapped'] = True
            os.close(dev_null)


    def __unmap_partitions(self):
        dev_null = os.open("/dev/null", os.O_WRONLY)
        for dev in self.disks.keys():
            d = self.disks[dev]
            if not d['mapped']:
                continue

            logging.debug("Removing compat symlinks")
            for pnum in d['partitions']:
                if self.partitions[pnum]['device'] != None:
                    os.unlink(self.partitions[pnum]['device'])
                    self.partitions[pnum]['device'] = None

            logging.debug("Unmapping %s" % d['disk'].device)
            rc = subprocess.call(["/sbin/kpartx", "-d", d['disk'].device],
                                 stdout=dev_null, stderr=dev_null)
            if rc != 0:
                os.close(dev_null)
                raise MountError("Failed to unmap partitions for '%s'" %
                                 d['disk'].device)

            d['mapped'] = False
            os.close(dev_null)


    def __calculate_mountorder(self):
        logging.debug("Calculating mount order")
        for p in self.partitions:
            self.mountOrder.append(p['mountpoint'])
            self.unmountOrder.append(p['mountpoint'])

        self.mountOrder.sort()
        self.unmountOrder.sort()
        self.unmountOrder.reverse()

    def cleanup(self):
        Mount.cleanup(self)
        self.__unmap_partitions()
        for dev in self.disks.keys():
            d = self.disks[dev]
            try:
                d['disk'].cleanup()
            except:
                pass

    def unmount(self):
        for mp in self.unmountOrder:
            if mp == 'swap':
                continue
            p = None
            for p1 in self.partitions:
                if p1['mountpoint'] == mp:
                    p = p1
                    break

            if p['mount'] != None:
                try:
                    p['mount'].cleanup()
                except:
                    pass
                p['mount'] = None

    def mount(self):
        for dev in self.disks.keys():
            d = self.disks[dev]
            d['disk'].create()

        self.__format_disks()
        self.__map_partitions()
        self.__calculate_mountorder()

        for mp in self.mountOrder:
            p = None
            for p1 in self.partitions:
                if p1['mountpoint'] == mp:
                    p = p1
                    break

            if mp == 'swap':
                subprocess.call(["/sbin/mkswap", p['device']])
                continue

            rmmountdir = False
            if p['mountpoint'] == "/":
                rmmountdir = True
            if p['fstype'] == "vfat" or p['fstype'] == "msdos":
                myDiskMount = VfatDiskMount
            elif p['fstype'] == "ext2" or p['fstype'] == "ext3":
                myDiskMount = ExtDiskMount
            else:
                raise MountError("Fail to support file system " + p['fstype'])

            pdisk = myDiskMount(RawDisk(p['size'] * 1024 * 1024, p['device']),
                                 self.mountdir + p['mountpoint'],
                                 p['fstype'],
                                 4096,
                                 p['mountpoint'],
                                 rmmountdir,
                                 self.skipformat)

            pdisk.mount()
            p['mount'] = pdisk

    def resparse(self, size = None):
        # Can't re-sparse a disk image - too hard
        pass

==> mic2_0.22.2.orig.tar.gz: mic2-0.22.2/mic/appcreate/partitionedfs.py <==
#
# partitionedfs.py: partitioned files system class, extends fs.py
#
# Copyright 2007-2008, Red Hat  Inc.
# Copyright 2008, Daniel P. Berrange
# Copyright 2008,  David P. Huff
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import os.path
import glob
import shutil
import subprocess
import logging

from mic.imgcreate.errors import *
from mic.imgcreate.fs import *


class PartitionedMount(Mount):
    def __init__(self, disks, mountdir, skipformat = False):
        Mount.__init__(self, mountdir)
        self.disks = {}
        for name in disks.keys():
            self.disks[name] = { 'disk': disks[name],  # Disk object
                                 'mapped': False, # True if kpartx mapping exists
                                 'numpart': 0, # Number of allocate partitions
                                 'partitions': [], # indexes to self.partitions
                                 'extended': 0, # Size of extended partition
                                 'offset': 0 } # Offset of next partition

        self.partitions = []
        self.mapped = False
        self.mountOrder = []
        self.unmountOrder = []
        self.parted=find_binary_path("parted")
        self.kpartx=find_binary_path("kpartx")
        self.mkswap=find_binary_path("mkswap")
        self.skipformat = skipformat

    def add_partition(self, size, disk, mountpoint, fstype = None, boot = False):
        self.partitions.append({'size': size,
                                'mountpoint': mountpoint, # Mount relative to chroot
                                'fstype': fstype,
                                'disk': disk,  # physical disk name holding partition
                                'device': None, # kpartx device node for partition
                                'mount': None, # Mount object
                                'num': None, # Partition number
                                'boot': boot}) # Bootable flag

    def __format_disks(self):
        dev_null = os.open("/dev/null", os.O_WRONLY)

        logging.debug("Assigning partitions to disks")
        for n in range(len(self.partitions)):
            p = self.partitions[n]

            if not self.disks.has_key(p['disk']):
                raise MountError("No disk %s for partition %s" % (p['disk'], p['mountpoint']))

            d = self.disks[p['disk']]
            d['numpart'] += 1
            if d['numpart'] > 3:
                # Increase allocation of extended partition to hold this partition
                d['extended'] += p['size']
                p['type'] = 'logical'
                p['num'] = d['numpart'] + 1
            else:
                p['type'] = 'primary'
                p['num'] = d['numpart']

            p['start'] = d['offset']
            d['offset'] += p['size']
            d['partitions'].append(n)
            logging.debug("Assigned %s to %s%d at %d at size %d" % (p['mountpoint'], p['disk'], p['num'], p['start'], p['size']))

        if self.skipformat:
            logging.debug("Skip disks format")
            return
        logging.debug("Formatting disks")
        for dev in self.disks.keys():
            d = self.disks[dev]
            logging.debug("Initializing partition table for %s" % (d['disk'].device))
            # FIXME
            rc = subprocess.call([self.parted, "-s", d['disk'].device, "mklabel", "msdos"],
                                 stdout = dev_null, stderr = dev_null)

            # XXX disabled return code check because parted always fails to
            # reload part table with loop devices. Annoying because we can't
            # distinguish this failure from real partition failures :-(
            if rc != 0 and 1 == 0:
                os.close(dev_null)
                raise MountError("Error writing partition table on %s" % d['disk'].device)

        # XXX we should probably work in cylinder units to keep fdisk happier..
        start = 0
        logging.debug("Creating partitions")
        for p in self.partitions:
            d = self.disks[p['disk']]
            if p['num'] == 5:
                logging.debug("Added extended part at %d of size %d" % (p['start'], d['extended']))
                rc = subprocess.call([self.parted, "-s", d['disk'].device, "mkpart", "extended",
                                     "%dM" % p['start'], "%dM" % (p['start'] + d['extended'])],
                                     stdout = dev_null, stderr = dev_null)

            logging.debug("Add %s part at %d of size %d" % (p['type'], p['start'], p['size']))
            rc = subprocess.call([self.parted, "-s", d['disk'].device, "mkpart",
                                 p['type'], "%dM" % p['start'], "%dM" % (p['start']+p['size'])],
                                 stdout = dev_null, stderr = dev_null)

            if p['boot']:
                rc = subprocess.call([self.parted, "-s", d['disk'].device, "set", "%d" % p['num'], "boot", "on"],
                                     stdout = dev_null, stderr = dev_null)
            # XXX disabled return code check because parted always fails to
            # reload part table with loop devices. Annoying because we can't
            # distinguish this failure from real partition failures :-(
            if rc != 0 and 1 == 0:
                os.close(dev_null)
                raise MountError("Error creating partition on %s" % d['disk'].device)
        os.close(dev_null)

    def __map_partitions(self):
        """Load it if dm_snapshot isn't loaded"""
        load_module("dm_snapshot")

        dev_null = os.open("/dev/null", os.O_WRONLY)
        for dev in self.disks.keys():
            d = self.disks[dev]
            if d['mapped']:
                continue

            logging.debug("Running kpartx on %s" % d['disk'].device )
            kpartx = subprocess.Popen([self.kpartx, "-l", "-v", d['disk'].device],
                                      stdout=subprocess.PIPE, stderr=dev_null)

            kpartxOutput = kpartx.communicate()[0].strip().split("\n")

            if kpartx.returncode:
                os.close(dev_null)
                raise MountError("Failed to query partition mapping for '%s'" %
                                 d['disk'].device)

            # Strip trailing blank and mask verbose output
            i = 0
            while i < len(kpartxOutput) and kpartxOutput[i][0:4] != "loop":
               i = i + 1
            kpartxOutput = kpartxOutput[i:]

            # Quick sanity check that the number of partitions matches
            # our expectation. If it doesn't, someone broke the code
            # further up
            if len(kpartxOutput) != d['numpart']:
                os.close(dev_null)
                raise MountError("Unexpected number of partitions from kpartx: %d != %d" %
                                 (len(kpartxOutput), d['numpart']))

            for i in range(len(kpartxOutput)):
                line = kpartxOutput[i]
                newdev = line.split()[0]
                mapperdev = "/dev/mapper/" + newdev
                loopdev = d['disk'].device + newdev[-1]

                logging.debug("Dev %s: %s -> %s" % (newdev, loopdev, mapperdev))
                pnum = d['partitions'][i]
                self.partitions[pnum]['device'] = loopdev

                # grub's install wants partitions to be named
                # to match their parent device + partition num
                # kpartx doesn't work like this, so we add compat
                # symlinks to point to /dev/mapper
                if os.path.exists(loopdev):
                    os.unlink(loopdev)
                os.symlink(mapperdev, loopdev)

            logging.debug("Adding partx mapping for %s" % d['disk'].device)
            rc = subprocess.call([self.kpartx, "-v", "-a", d['disk'].device],
                                 stdout=dev_null, stderr=dev_null)
            if rc != 0:
                os.close(dev_null)
                raise MountError("Failed to map partitions for '%s'" %
                                 d['disk'].device)
            d['mapped'] = True
            os.close(dev_null)


    def __unmap_partitions(self):
        dev_null = os.open("/dev/null", os.O_WRONLY)
        for dev in self.disks.keys():
            d = self.disks[dev]
            if not d['mapped']:
                continue

            logging.debug("Removing compat symlinks")
            for pnum in d['partitions']:
                if self.partitions[pnum]['device'] != None:
                    os.unlink(self.partitions[pnum]['device'])
                    self.partitions[pnum]['device'] = None

            logging.debug("Unmapping %s" % d['disk'].device)
            rc = subprocess.call([self.kpartx, "-d", d['disk'].device],
                                 stdout=dev_null, stderr=dev_null)
            if rc != 0:
                os.close(dev_null)
                raise MountError("Failed to unmap partitions for '%s'" %
                                 d['disk'].device)

            d['mapped'] = False
            os.close(dev_null)


    def __calculate_mountorder(self):
        logging.debug("Calculating mount order")
        for p in self.partitions:
            self.mountOrder.append(p['mountpoint'])
            self.unmountOrder.append(p['mountpoint'])

        self.mountOrder.sort()
        self.unmountOrder.sort()
        self.unmountOrder.reverse()

    def cleanup(self):
        Mount.cleanup(self)
        self.__unmap_partitions()
        for dev in self.disks.keys():
            d = self.disks[dev]
            try:
                d['disk'].cleanup()
            except:
                pass

    def unmount(self):
        for mp in self.unmountOrder:
            if mp == 'swap':
                continue
            p = None
            for p1 in self.partitions:
                if p1['mountpoint'] == mp:
                    p = p1
                    break

            if p['mount'] != None:
                try:
                    p['mount'].cleanup()
                except:
                    pass
                p['mount'] = None

    def mount(self):
        for dev in self.disks.keys():
            d = self.disks[dev]
            d['disk'].create()

        self.__format_disks()
        self.__map_partitions()
        self.__calculate_mountorder()

        for mp in self.mountOrder:
            p = None
            for p1 in self.partitions:
                if p1['mountpoint'] == mp:
                    p = p1
                    break

            if mp == 'swap':
                subprocess.call([self.mkswap, p['device']])
                continue

            rmmountdir = False
            if p['mountpoint'] == "/":
                rmmountdir = True
            if p['fstype'] == "vfat" or p['fstype'] == "msdos":
                myDiskMount = VfatDiskMount
            elif p['fstype'] in ("ext2", "ext3", "ext4"):
                myDiskMount = ExtDiskMount
            elif p['fstype'] == "btrfs":
                myDiskMount = BtrfsDiskMount
            else:
                raise MountError("Fail to support file system " + p['fstype'])

            pdisk = myDiskMount(RawDisk(p['size'] * 1024 * 1024, p['device']),
                                 self.mountdir + p['mountpoint'],
                                 p['fstype'],
                                 4096,
                                 p['mountpoint'],
                                 rmmountdir,
                                 self.skipformat)

            pdisk.mount()
            p['mount'] = pdisk

    def resparse(self, size = None):
        # Can't re-sparse a disk image - too hard
        pass
